/*====================================================================
* Project:
* Developed using:
* Function: getting started sample with ASC0, interrupt and timer
*           functions for rs232 (TC1796)
*
* Copyright HighTec EDV-Systeme GmbH 1982-2007
*====================================================================*/

#include <machine/wdtcon.h>
#include <machine/cint.h>

#include <tc1796b/port5-struct.h>
#include <tc1796b/asc-struct.h>

#include "rs232.h"

#define RX_EMPTY	0x1
#define TX_EMPTY	0x2

#define BAUD		9600

#define TRANSMIT_INTERRUPT	2
#define RECEIVE_INTERRUPT	4

/* ASC Modes */
#define ASCM_8SYNC		0	/* 8 Bit Data Sync. */
#define ASCM_8ASYNC		1	/* 8 Bit Data Async. */
#define ASCM_8IRDA		2	/* 8 Bit Data IrDA */
#define ASCM_7P			3	/* 7 Bit Data + Parity */
#define ASCM_9			4	/* 9 Bit Data */
#define ASCM_8W			5	/* 8 Bit Data + Wake up Bit */
#define ASCM_8P			7	/* 8 Bit Data + Parity */

/* Port Modes */
#define IN_NOPULL0		0x0	/* Port Input No Pull Device */
#define IN_PULLDOWN		0x1	/* Port Input Pull Down Device */
#define IN_PULLUP		0x2	/* Port Input Pull Up Device */
#define IN_NOPULL3		0x3	/* Port Input No Pull Device */
#define OUT_PPGPIO		0x8	/* Port Output General Purpose Push/Pull */
#define OUT_PPALT1		0x9 /* Port Output Alternate 1 Function Push/Pull */
#define OUT_PPALT2		0xA	/* Port Output Alternate 2 Function Push/Pull */
#define OUT_PPALT3		0xB	/* Port Output Alternate 3 Function Push/Pull */
#define OUT_ODGPIO		0xC	/* Port Output General Purpose Open Drain */
#define OUT_ODALT1		0xD	/* Port Output Alternate 1 Function Open Drain */
#define OUT_ODALT2		0xE	/* Port Output Alternate 2 Function Open Drain */
#define OUT_ODALT3		0xF	/* Port Output Alternate 3 Function Open Drain */


static volatile unsigned int Status = 0;

/*
	simple example for locating functions in internal program memory
 */
#pragma section .internalcode x
/* transmit interrupt routine */
void transmit_irq(void)
{
	/* setting status to tx is empty */
	Status |= TX_EMPTY;
}

/* receive interrupt routine */
void receive_irq(void)
{
	/* setting Status to rx is not empty */
	Status &= ~(RX_EMPTY);
}
#pragma section


/* initialize ASC0 */
void init_rs232(unsigned int frequency)
{
	ASC_t   *asc  = (ASC_t *) ASC0_BASE;
	PORT5_t *port = (PORT5_t *) P5_BASE;

	/* initialize vector and trap tables */
	_init_vectab();

	/* Install handlers for transmit and receive interrupts */
	_install_int_handler(TRANSMIT_INTERRUPT, (void (*) (int)) transmit_irq, 0);
	_install_int_handler(RECEIVE_INTERRUPT,  (void (*) (int)) receive_irq,  0);

	/* Set TXD to "output" and "high" */
	/* set P5.1 to output and high and drive activ */
	port->IOCR0.bits.PC1 = OUT_PPALT1;
	port->OMR.bits.PS1 = 1;

	/* initialize ASC0 */
	/* first must set clock for ASC0 and enable module */
	/* register is endinit protected */
	/* password access to wdtcon0 */
	unlock_wdtcon();
	asc->CLC.bits.RMC = 1;
	asc->CLC.bits.DISR = 0;
	/* setting endinit, password access to wdtcon0 */
	lock_wdtcon();

	/* clear control register */
	asc->CON.reg = 0;
	/* setting Baudrate */
	asc->BG.reg = frequency / (32 * BAUD) - 1;
	/* setting Fractional divider */
	asc->FDV.reg = 0;
	/* setting transmit buffer sr to pn 2 */
	asc->TSRC.reg = ASCn_TSRC_SRE_MASK | TRANSMIT_INTERRUPT;
	/* setting receive sr to pn 4 */
	asc->RSRC.reg = ASCn_RSRC_SRE_MASK | RECEIVE_INTERRUPT;

	asc->CON.bits.M = ASCM_8ASYNC;
	asc->CON.bits.R = 1;
	asc->CON.bits.REN = 1;

	Status = TX_EMPTY | RX_EMPTY;
}

/* send a string to ASC0 */
void message(const char *Message)
{
	ASC_t   *asc  = (ASC_t *) ASC0_BASE;

	/* while the character is not 0 */
	while (*Message)
	{
		/* wait until TX is empty */
		if (Status & TX_EMPTY)
		{
			/* set TX to not empty */
			Status &= ~TX_EMPTY;
			/* write character to ASC0 */
			asc->TBUF.bits.TD_VALUE = *Message++;
		}
	}
}
